@@ -17,6 +17,7 @@ import Hledger.Web.Widget.Common (accountQuery, mixedAmountAsHtml)
 -- | The formatted journal view, with sidebar.
 getJournalR :: Handler Html
 getJournalR = do
+  checkServerSideUiEnabled
   VD{caps, j, m, opts, qopts, today} <- getViewData
   when (CapView `notElem` caps) (permissionDenied "Missing the 'view' capability")
   let title = case inAccount qopts of
@@ -24,7 +25,7 @@ getJournalR = do
         Just (a, inclsubs) -> "Transactions in " <> a <> if inclsubs then "" else " (excluding subaccounts)"
       title' = title <> if m /= Any then ", filtered" else ""
       acctlink a = (RegisterR, [("q", accountQuery a)])
-      (_, items) = journalTransactionsReport (reportopts_ $ cliopts_ opts) j m
+      (_, items) = transactionsReport (reportopts_ $ cliopts_ opts) j m
 
   defaultLayout $ do
     setTitle "journal - hledger-web"
