@@ -4,6 +4,7 @@ module Hledger.Web.WebOptions where
 
 import Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as BC
+import Data.ByteString.UTF8 (fromString)
 import Data.CaseInsensitive (CI, mk)
 import Control.Monad (join)
 import Data.Default (Default(def))
@@ -11,6 +12,8 @@ import Data.Maybe (fromMaybe)
 import qualified Data.Text as T
 import Data.Text (Text)
 import System.Environment (getArgs)
+import Network.Wai as WAI
+import Network.Wai.Middleware.Cors
 
 import Hledger.Cli hiding (progname, version)
 import Hledger.Web.Settings (defhost, defport, defbaseurl)
@@ -25,12 +28,21 @@ version = ""
 prognameandversion :: String
 prognameandversion = progname ++ " " ++ version :: String
 
-webflags :: [Flag [(String, String)]]
+webflags :: [Flag RawOpts]
 webflags =
   [ flagNone
       ["serve", "server"]
       (setboolopt "serve")
       "serve and log requests, don't browse or auto-exit"
+  , flagNone
+      ["serve-api"]
+      (setboolopt "serve-api")
+      "like --serve, but serve only the JSON web API, without the server-side web UI"
+  , flagReq
+      ["cors"]
+      (\s opts -> Right $ setopt "cors" s opts)
+      "ORIGIN"
+      ("allow cross-origin requests from the specified origin; setting ORIGIN to \"*\" allows requests from any origin")
   , flagReq
       ["host"]
       (\s opts -> Right $ setopt "host" s opts)
@@ -63,11 +75,11 @@ webflags =
       "read capabilities to enable from a HTTP header, like X-Sandstorm-Permissions (default: disabled)"
   ]
 
-webmode :: Mode [(String, String)]
+webmode :: Mode RawOpts
 webmode =
   (mode
      "hledger-web"
-     [("command", "web")]
+     (setopt "command" "web" def)
      "start serving the hledger web interface"
      (argsFlag "[PATTERNS]")
      [])
@@ -75,6 +87,7 @@ webmode =
       Group
       { groupUnnamed = webflags
       , groupHidden =
+          hiddenflags ++
           [ flagNone
               ["binary-filename"]
               (setboolopt "binary-filename")
@@ -88,6 +101,8 @@ webmode =
 -- hledger-web options, used in hledger-web and above
 data WebOpts = WebOpts
   { serve_ :: Bool
+  , serve_api_ :: Bool
+  , cors_ :: Maybe String
   , host_ :: String
   , port_ :: Int
   , base_url_ :: String
@@ -98,7 +113,7 @@ data WebOpts = WebOpts
   } deriving (Show)
 
 defwebopts :: WebOpts
-defwebopts = WebOpts def def def def def [CapView, CapAdd] Nothing def
+defwebopts = WebOpts def def Nothing def def def def [CapView, CapAdd] Nothing def
 
 instance Default WebOpts where def = defwebopts
 
@@ -119,6 +134,8 @@ rawOptsToWebOpts rawopts =
     return
       defwebopts
       { serve_ = boolopt "serve" rawopts
+      , serve_api_ = boolopt "serve-api" rawopts
+      , cors_ = maybestringopt "cors" rawopts
       , host_ = h
       , port_ = p
       , base_url_ = b
@@ -140,7 +157,7 @@ checkWebOpts wopts = do
 getHledgerWebOpts :: IO WebOpts
 getHledgerWebOpts = do
   args <- fmap replaceNumericFlags . expandArgsAt =<< getArgs
-  rawOptsToWebOpts . decodeRawOpts . either usageError id $ process webmode args
+  rawOptsToWebOpts . either usageError id $ process webmode args
 
 data Capability
   = CapView
@@ -159,3 +176,21 @@ capabilityFromBS "view" = Right CapView
 capabilityFromBS "add" = Right CapAdd
 capabilityFromBS "manage" = Right CapManage
 capabilityFromBS x = Left x
+
+simplePolicyWithOrigin :: Origin -> CorsResourcePolicy
+simplePolicyWithOrigin origin =
+    simpleCorsResourcePolicy { corsOrigins = Just ([origin], False) }
+
+
+corsPolicyFromString :: String -> WAI.Middleware
+corsPolicyFromString origin =
+  let
+    policy = case origin of
+        "*" -> simpleCorsResourcePolicy
+        url -> simplePolicyWithOrigin $ fromString url
+  in
+    cors (const $ Just policy)
+
+corsPolicy :: WebOpts -> (Application -> Application)
+corsPolicy opts =
+  maybe id corsPolicyFromString $ cors_ opts
