@@ -13,14 +13,15 @@ module Hledger.Web.Widget.AddForm
 
 import Control.Monad.State.Strict (evalStateT)
 import Data.Bifunctor (first)
-import Data.List (dropWhileEnd, nub, sort, unfoldr)
+import Data.List (dropWhileEnd, intercalate, nub, sort, unfoldr)
 import Data.Maybe (isJust)
+#if !(MIN_VERSION_base(4,13,0))
 import Data.Semigroup ((<>))
+#endif
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time (Day)
 import Text.Blaze.Internal (Markup, preEscapedString)
-import Text.JSON
 import Text.Megaparsec (bundleErrors, eof, parseErrorTextPretty, runParser)
 import Yesod
 
@@ -69,13 +70,12 @@ addForm j today = identifyForm "add" $ \extra -> do
   (amtRes, _) <- mreq listField amtFS Nothing
   let (postRes, displayRows) = validatePostings acctRes amtRes
 
+  -- bindings used in add-form.hamlet
   let descriptions = sort $ nub $ tdescription <$> jtxns j
-      escapeJSSpecialChars = regexReplaceCI "</script>" "<\\/script>" -- #236
-      listToJsonValueObjArrayStr = preEscapedString . escapeJSSpecialChars .
-        encode . JSArray . fmap (\a -> JSObject $ toJSObject [("value", showJSON a)])
       journals = fst <$> jfiles j
 
   pure (validateTransaction dateRes descRes postRes, $(widgetFile "add-form"))
+
   where
     dateFS = FieldSettings "date" Nothing Nothing (Just "date")
       [("class", "form-control input-lg"), ("placeholder", "Date")]
@@ -94,6 +94,28 @@ addForm j today = identifyForm "add" $ \extra -> do
       , fieldEnctype = UrlEncoded
       }
 
+    -- Used in add-form.hamlet
+    toBloodhoundJson :: [Text] -> Markup
+    toBloodhoundJson ts =
+      -- This used to work, but since 1.16, it seems like something changed.
+      -- toJSON ("a"::Text) gives String "a" instead of "a", etc.
+      -- preEscapedString . escapeJSSpecialChars . show . toJSON
+      preEscapedString $ concat [
+        "[",
+        intercalate "," $ map (
+          ("{\"value\":" ++).
+          (++"}").
+          escapeJSSpecialChars .
+          drop 7 .  -- "String "
+          show .
+          toJSON
+          ) ts,
+        "]"
+        ]
+      where
+        -- avoid https://github.com/simonmichael/hledger/issues/236
+        escapeJSSpecialChars = regexReplaceCI "</script>" "<\\/script>"
+
 validateTransaction ::
      FormResult Day
   -> FormResult Text
